package envoy.client.event;

import java.io.IOException;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>EnvoyLogger.java</strong><br>
 * Created: <strong>14 Dec 2019</strong><br>
 * 
 * @author Leon Hofmeister
 * @since Envoy v0.2-alpha
 */
public class EnvoyLogger extends Logger {

	private Logger	logger;
	private Level	fileLevelBarrier	= Level.CONFIG;

	public EnvoyLogger(String name) {
		super(name, null);
		try {
			SimpleFormatter	formatter	= new SimpleFormatter();
			FileHandler		fh			= new FileHandler("envoy_user.log");
			fh.setLevel(fileLevelBarrier);
			fh.setFormatter(formatter);
			ConsoleHandler ch = new ConsoleHandler();
			ch.setLevel(Level.FINEST);
			ch.setFormatter(formatter);
			logger.addHandler(fh);
			logger.addHandler(ch);
		} catch (IOException | SecurityException e) {
			e.printStackTrace();
			this.log(Level.FINE, "Ironically, the logger encountered an error while initialising. That certainly needs to be logged :)");
		}
	}

	/**
	 * Logs a message. If the problem severity is above the FileLevel-barrier, then
	 * the log entry will be written to a file. Regardless of problem severity,
	 * everything will be printed to the console.
	 * 
	 * @param level the problem severity
	 * @param msg   the message to be written in the log
	 * @since Envoy v0.2-alpha
	 */
	@Override
	public void log(Level level, String msg) {
		LogRecord lr = new LogRecord(level, msg);
		logger.log(lr);
	}

	/**
	 * Logs a message. If the problem severity is above the {@code FileLevel}
	 * barrier, then the log entry will be written to a file.
	 * Regardless of problem severity, everything will be printed to the console.
	 * 
	 * @param logRecord the LogRecord (Level and String) to be treated by the
	 *                  Logger.
	 * @since Envoy v0.2-alpha
	 */
	@Override
	public void log(LogRecord logRecord) { logger.log(logRecord); }

	/**
	 * @return the fileLevelBarrier: The current barrier for writing logs to a file.
	 * @since Envoy v0.2-alpha
	 */
	public Level getFileLevelBarrier() { return fileLevelBarrier; }

	/**
	 * @param fileLevelBarrier the severity below which {@link LogRecord}s will be
	 *                         written only to the console. At or above they'll also
	 *                         be logged in a file. Can be written either in Digits
	 *                         from 0 - 1000 or with the according name of the level
	 * @since Envoy v0.2-alpha
	 */
	public void setFileLevel(String fileLevelBarrier) { this.fileLevelBarrier = Level.parse(fileLevelBarrier); }
}
